"""Top-level import package for {{project_slug}}"""
from {{project_slug}}.log import LoggerConfig


# Package meta information
__version__ = '{{project_version}}'
# Logger configuration can be accessed through global variable LOGCONFIG
LOGCONFIG = LoggerConfig()
